@@ -77,7 +77,7 @@ typedef struct {
 static int crypt_sodium_init_(cryptstate_T *state, char_u *key, crypt_arg_T *arg);
 static long crypt_sodium_buffer_decode(cryptstate_T *state, char_u *from, size_t len, char_u **buf_out, int last);
 static long crypt_sodium_buffer_encode(cryptstate_T *state, char_u *from, size_t len, char_u **buf_out, int last);
-# if defined(FEAT_SODIUM)
+#if defined(FEAT_SODIUM)
 static void crypt_long_long_to_char(long long n, char_u *s);
 static void crypt_int_to_char(int n, char_u *s);
 static long long crypt_char_to_long_long(char_u *s);
@@ -924,7 +924,7 @@ crypt_sodium_init_(
     char_u		*key UNUSED,
     crypt_arg_T		*arg UNUSED)
 {
-# ifdef FEAT_SODIUM
+#ifdef FEAT_SODIUM
     // crypto_box_SEEDBYTES ==  crypto_secretstream_xchacha20poly1305_KEYBYTES
     unsigned char	dkey[crypto_box_SEEDBYTES]; // 32
     sodium_state_T	*sd_state;
@@ -946,14 +946,14 @@ crypt_sodium_init_(
 	memlimit = crypto_pwhash_MEMLIMIT_INTERACTIVE;
 	alg = crypto_pwhash_ALG_DEFAULT;
 
-#if 0
+# if 0
 	// For testing
 	if (state->method_nr == CRYPT_M_SOD2)
 	{
 	    opslimit = crypto_pwhash_OPSLIMIT_MODERATE;
 	    memlimit = crypto_pwhash_MEMLIMIT_MODERATE;
 	}
-#endif
+# endif
 
 	// derive a key from the password
 	if (crypto_pwhash(dkey, sizeof(dkey), (const char *)key, STRLEN(key),
@@ -1017,12 +1017,12 @@ crypt_sodium_init_(
 	alg = crypt_char_to_int(p);
 	p += sizeof(alg);
 
-#ifdef FEAT_EVAL
+# ifdef FEAT_EVAL
 	crypt_sodium_report_hash_params(opslimit,
 					    crypto_pwhash_OPSLIMIT_INTERACTIVE,
 		(size_t)memlimit, crypto_pwhash_MEMLIMIT_INTERACTIVE,
 		alg, crypto_pwhash_ALG_DEFAULT);
-#endif
+# endif
 
 	if (crypto_pwhash(dkey, sizeof(dkey), (const char *)key, STRLEN(key),
 			  arg->cat_salt, opslimit, (size_t)memlimit, alg) != 0)
@@ -1047,10 +1047,10 @@ crypt_sodium_init_(
     state->method_state = sd_state;
 
     return OK;
-# else
+#else
     emsg(_(e_libsodium_not_built_in));
     return FAIL;
-# endif
+#endif
 }
 
 /*
@@ -1190,7 +1190,7 @@ crypt_sodium_buffer_encode(
     char_u	**buf_out UNUSED,
     int		last UNUSED)
 {
-# ifdef FEAT_SODIUM
+#ifdef FEAT_SODIUM
     // crypto_box_SEEDBYTES ==  crypto_secretstream_xchacha20poly1305_KEYBYTES
     unsigned long long	out_len;
     char_u		*ptr;
@@ -1223,9 +1223,9 @@ crypt_sodium_buffer_encode(
     sod_st->count++;
     return out_len + (first
 		      ? crypto_secretstream_xchacha20poly1305_HEADERBYTES : 0);
-# else
+#else
     return -1;
-# endif
+#endif
 }
 
 /*
@@ -1240,7 +1240,7 @@ crypt_sodium_buffer_decode(
     char_u	**buf_out UNUSED,
     int		last UNUSED)
 {
-# ifdef FEAT_SODIUM
+#ifdef FEAT_SODIUM
     // crypto_box_SEEDBYTES ==  crypto_secretstream_xchacha20poly1305_KEYBYTES
     sodium_state_T *sod_st = state->method_state;
     unsigned char  tag;
@@ -1282,12 +1282,12 @@ crypt_sodium_buffer_decode(
     if (tag == crypto_secretstream_xchacha20poly1305_TAG_FINAL && !last)
 	emsg(_(e_libsodium_decryption_failed_premature));
     return (long) out_len;
-# else
+#else
     return -1;
-# endif
+#endif
 }
 
-# if defined(FEAT_SODIUM)
+#if defined(FEAT_SODIUM)
     void
 crypt_sodium_lock_key(char_u *key)
 {
@@ -1319,7 +1319,7 @@ crypt_sodium_randombytes_random(void)
     return randombytes_random();
 }
 
-#if defined(FEAT_EVAL)
+# if defined(FEAT_EVAL)
     static void
 crypt_sodium_report_hash_params(
 	unsigned long long opslimit,
@@ -1347,7 +1347,7 @@ crypt_sodium_report_hash_params(
 	verbose_leave();
     }
 }
-#endif
+# endif
 
     static void
 crypt_long_long_to_char(long long n, char_u *s)
@@ -1356,7 +1356,7 @@ crypt_long_long_to_char(long long n, char_u *s)
     for (i = 0; i < 8; i++)
     {
 	s[i] = (char_u)(n & 0xff);
-	n = (unsigned)n >> 8;
+	n = (unsigned long long)n >> 8;
     }
 }
 
@@ -1367,7 +1367,7 @@ crypt_int_to_char(int n, char_u *s)
     for (i = 0; i < 4; i++)
     {
 	s[i] = (char_u)(n & 0xff);
-	n = (unsigned)n >> 8;
+	n = (unsigned long long)n >> 8;
     }
 }
 
@@ -1405,6 +1405,6 @@ crypt_char_to_int(char_u *s)
     }
     return retval;
 }
-# endif
+#endif
 
 #endif // FEAT_CRYPT
