@@ -104,7 +104,7 @@ win_id2wp_tp(int id, tabpage_T **tpp)
 		*tpp = tp;
 	    return wp;
 	}
-#ifdef FEAT_PROP_POPUP
+# ifdef FEAT_PROP_POPUP
     // popup windows are in separate lists
     FOR_ALL_TABPAGES(tp)
 	FOR_ALL_POPUPWINS_IN_TAB(tp, wp)
@@ -121,7 +121,7 @@ win_id2wp_tp(int id, tabpage_T **tpp)
 		*tpp = curtab;  // any tabpage would do
 	    return wp;
 	}
-#endif
+# endif
 
     return NULL;
 }
@@ -184,7 +184,7 @@ find_win_by_nr(
     }
     if (nr >= LOWEST_WIN_ID)
     {
-#ifdef FEAT_PROP_POPUP
+# ifdef FEAT_PROP_POPUP
 	// check tab-local popup windows
 	for (wp = (tp == NULL ? curtab : tp)->tp_first_popupwin;
 						   wp != NULL; wp = wp->w_next)
@@ -194,7 +194,7 @@ find_win_by_nr(
 	FOR_ALL_POPUPWINS(wp)
 	    if (wp->w_id == nr)
 		return wp;
-#endif
+# endif
 	return NULL;
     }
     return wp;
@@ -416,26 +416,27 @@ get_win_info(win_T *wp, short tpnr, short winnr)
     dict_add_number(dict, "winnr", winnr);
     dict_add_number(dict, "winid", wp->w_id);
     dict_add_number(dict, "height", wp->w_height);
+    dict_add_number(dict, "status_height", wp->w_status_height);
     dict_add_number(dict, "winrow", wp->w_winrow + 1);
     dict_add_number(dict, "topline", wp->w_topline);
     dict_add_number(dict, "botline", wp->w_botline - 1);
-#ifdef FEAT_MENU
+# ifdef FEAT_MENU
     dict_add_number(dict, "winbar", wp->w_winbar_height);
-#endif
+# endif
     dict_add_number(dict, "width", wp->w_width);
     dict_add_number(dict, "wincol", wp->w_wincol + 1);
     dict_add_number(dict, "textoff", win_col_off(wp));
     dict_add_number(dict, "bufnr", wp->w_buffer->b_fnum);
     dict_add_number(dict, "leftcol", wp->w_leftcol);
 
-#ifdef FEAT_TERMINAL
+# ifdef FEAT_TERMINAL
     dict_add_number(dict, "terminal", bt_terminal(wp->w_buffer));
-#endif
-#ifdef FEAT_QUICKFIX
+# endif
+# ifdef FEAT_QUICKFIX
     dict_add_number(dict, "quickfix", bt_quickfix(wp->w_buffer));
     dict_add_number(dict, "loclist",
 		      (bt_quickfix(wp->w_buffer) && wp->w_llist_ref != NULL));
-#endif
+# endif
 
     // Add a reference to window variables
     dict_add_dict(dict, "variables", wp->w_vars);
@@ -555,7 +556,7 @@ f_getwininfo(typval_T *argvars, typval_T *rettv)
 		return;
 	}
     }
-#ifdef FEAT_PROP_POPUP
+# ifdef FEAT_PROP_POPUP
     if (wparg != NULL)
     {
 	tabnr = 0;
@@ -571,7 +572,7 @@ found:
 	if (d != NULL)
 	    list_append_dict(rettv->vval.v_list, d);
     }
-#endif
+# endif
 }
 
 /*
@@ -589,7 +590,7 @@ f_getwinpos(typval_T *argvars UNUSED, typval_T *rettv)
     if (in_vim9script() && check_for_opt_number_arg(argvars, 0) == FAIL)
 	return;
 
-#if defined(FEAT_GUI) \
+# if defined(FEAT_GUI) \
 	|| (defined(HAVE_TGETENT) && defined(FEAT_TERMRESPONSE)) \
 	|| defined(MSWIN)
     {
@@ -600,7 +601,7 @@ f_getwinpos(typval_T *argvars UNUSED, typval_T *rettv)
 
 	(void)ui_get_winpos(&x, &y, timeout);
     }
-#endif
+# endif
     list_append_number(rettv->vval.v_list, (varnumber_T)x);
     list_append_number(rettv->vval.v_list, (varnumber_T)y);
 }
@@ -613,7 +614,7 @@ f_getwinpos(typval_T *argvars UNUSED, typval_T *rettv)
 f_getwinposx(typval_T *argvars UNUSED, typval_T *rettv)
 {
     rettv->vval.v_number = -1;
-#if defined(FEAT_GUI) \
+# if defined(FEAT_GUI) \
 	|| (defined(HAVE_TGETENT) && defined(FEAT_TERMRESPONSE)) \
 	|| defined(MSWIN)
 
@@ -623,7 +624,7 @@ f_getwinposx(typval_T *argvars UNUSED, typval_T *rettv)
 	if (ui_get_winpos(&x, &y, 100) == OK)
 	    rettv->vval.v_number = x;
     }
-#endif
+# endif
 }
 
 /*
@@ -633,7 +634,7 @@ f_getwinposx(typval_T *argvars UNUSED, typval_T *rettv)
 f_getwinposy(typval_T *argvars UNUSED, typval_T *rettv)
 {
     rettv->vval.v_number = -1;
-#if defined(FEAT_GUI) \
+# if defined(FEAT_GUI) \
 	|| (defined(HAVE_TGETENT) && defined(FEAT_TERMRESPONSE)) \
 	|| defined(MSWIN)
     {
@@ -642,7 +643,7 @@ f_getwinposy(typval_T *argvars UNUSED, typval_T *rettv)
 	if (ui_get_winpos(&x, &y, 100) == OK)
 	    rettv->vval.v_number = y;
     }
-#endif
+# endif
 }
 
 /*
@@ -728,10 +729,11 @@ f_win_execute(typval_T *argvars, typval_T *rettv)
     pos_T	curpos = wp->w_cursor;
     char_u	cwd[MAXPATHL];
     int	cwd_status = FAIL;
-#ifdef FEAT_AUTOCHDIR
+# ifdef FEAT_AUTOCHDIR
     char_u	autocwd[MAXPATHL];
     int	apply_acd = FALSE;
-#endif
+    char_u	*save_sfname = NULL;
+# endif
 
     // Getting and setting directory can be slow on some systems, only do
     // this when the current or target window/tab have a local directory or
@@ -742,22 +744,24 @@ f_win_execute(typval_T *argvars, typval_T *rettv)
 		|| (curtab != tp
 		    && (curtab->tp_localdir != NULL
 			|| tp->tp_localdir != NULL))
-#ifdef FEAT_AUTOCHDIR
+# ifdef FEAT_AUTOCHDIR
 		|| p_acd
-#endif
+# endif
 	       ))
 	cwd_status = mch_dirname(cwd, MAXPATHL);
 
-#ifdef FEAT_AUTOCHDIR
+# ifdef FEAT_AUTOCHDIR
     // If 'acd' is set, check we are using that directory.  If yes, then
     // apply 'acd' afterwards, otherwise restore the current directory.
     if (cwd_status == OK && p_acd)
     {
+	if (curbuf->b_sfname != NULL && curbuf->b_fname == curbuf->b_sfname)
+	    save_sfname = vim_strsave(curbuf->b_sfname);
 	do_autochdir();
 	apply_acd = mch_dirname(autocwd, MAXPATHL) == OK
 	    && STRCMP(cwd, autocwd) == 0;
     }
-#endif
+# endif
 
     if (switch_win_noblock(&switchwin, wp, tp, TRUE) == OK)
     {
@@ -765,13 +769,26 @@ f_win_execute(typval_T *argvars, typval_T *rettv)
 	execute_common(argvars, rettv, 1);
     }
     restore_win_noblock(&switchwin, TRUE);
-#ifdef FEAT_AUTOCHDIR
+# ifdef FEAT_AUTOCHDIR
     if (apply_acd)
+    {
+	vim_free(save_sfname);
 	do_autochdir();
+    }
     else
-#endif
+# endif
 	if (cwd_status == OK)
+	{
 	    mch_chdir((char *)cwd);
+# ifdef FEAT_AUTOCHDIR
+	    if (save_sfname != NULL)
+	    {
+		vim_free(curbuf->b_sfname);
+		curbuf->b_sfname = save_sfname;
+		curbuf->b_fname = curbuf->b_sfname;
+	    }
+# endif
+	}
 
     // Update the status line if the cursor moved.
     if (win_valid(wp) && !EQUAL_POS(curpos, wp->w_cursor))
@@ -835,13 +852,13 @@ f_win_gotoid(typval_T *argvars, typval_T *rettv)
 
     if (text_or_buf_locked())
 	return;
-#if defined(FEAT_PROP_POPUP) && defined(FEAT_TERMINAL)
+# if defined(FEAT_PROP_POPUP) && defined(FEAT_TERMINAL)
     if (popup_is_popup(curwin) && curbuf->b_term != NULL)
     {
 	emsg(_(e_not_allowed_for_terminal_in_popup_window));
 	return;
     }
-#endif
+# endif
     FOR_ALL_TAB_WINDOWS(tp, wp)
 	if (wp->w_id == id)
 	{
@@ -1048,21 +1065,21 @@ f_win_gettype(typval_T *argvars, typval_T *rettv)
     }
     if (is_aucmd_win(wp))
 	rettv->vval.v_string = vim_strsave((char_u *)"autocmd");
-#if defined(FEAT_QUICKFIX)
+# if defined(FEAT_QUICKFIX)
     else if (wp->w_p_pvw)
 	rettv->vval.v_string = vim_strsave((char_u *)"preview");
-#endif
-#ifdef FEAT_PROP_POPUP
+# endif
+# ifdef FEAT_PROP_POPUP
     else if (WIN_IS_POPUP(wp))
 	rettv->vval.v_string = vim_strsave((char_u *)"popup");
-#endif
+# endif
     else if (wp == cmdwin_win)
 	rettv->vval.v_string = vim_strsave((char_u *)"command");
-#ifdef FEAT_QUICKFIX
+# ifdef FEAT_QUICKFIX
     else if (bt_quickfix(wp->w_buffer))
 	rettv->vval.v_string = vim_strsave((char_u *)
 		(wp->w_llist_ref != NULL ? "loclist" : "quickfix"));
-#endif
+# endif
 
 }
 
@@ -1199,10 +1216,14 @@ f_winrestcmd(typval_T *argvars UNUSED, typval_T *rettv)
 	winnr = 1;
 	FOR_ALL_WINDOWS(wp)
 	{
-	    sprintf((char *)buf, ":%dresize %d|", winnr, wp->w_height);
-	    ga_concat(&ga, buf);
-	    sprintf((char *)buf, "vert :%dresize %d|", winnr, wp->w_width);
-	    ga_concat(&ga, buf);
+	    size_t  buflen;
+
+	    buflen = vim_snprintf_safelen((char *)buf, sizeof(buf),
+		":%dresize %d|", winnr, wp->w_height);
+	    ga_concat_len(&ga, buf, buflen);
+	    buflen = vim_snprintf_safelen((char *)buf, sizeof(buf),
+		"vert :%dresize %d|", winnr, wp->w_width);
+	    ga_concat_len(&ga, buf, buflen);
 	    ++winnr;
 	}
     }
@@ -1238,10 +1259,10 @@ f_winrestview(typval_T *argvars, typval_T *rettv UNUSED)
 
     if (dict_has_key(dict, "topline"))
 	set_topline(curwin, (linenr_T)dict_get_number(dict, "topline"));
-#ifdef FEAT_DIFF
+# ifdef FEAT_DIFF
     if (dict_has_key(dict, "topfill"))
 	curwin->w_topfill = (int)dict_get_number(dict, "topfill");
-#endif
+# endif
     if (dict_has_key(dict, "leftcol"))
 	curwin->w_leftcol = (colnr_T)dict_get_number(dict, "leftcol");
     if (dict_has_key(dict, "skipcol"))
@@ -1256,9 +1277,9 @@ f_winrestview(typval_T *argvars, typval_T *rettv UNUSED)
 	curwin->w_topline = 1;
     if (curwin->w_topline > curbuf->b_ml.ml_line_count)
 	curwin->w_topline = curbuf->b_ml.ml_line_count;
-#ifdef FEAT_DIFF
+# ifdef FEAT_DIFF
     check_topfill(curwin, TRUE);
-#endif
+# endif
 }
 
 /*
@@ -1280,9 +1301,9 @@ f_winsaveview(typval_T *argvars UNUSED, typval_T *rettv)
     dict_add_number(dict, "curswant", (long)curwin->w_curswant);
 
     dict_add_number(dict, "topline", (long)curwin->w_topline);
-#ifdef FEAT_DIFF
+# ifdef FEAT_DIFF
     dict_add_number(dict, "topfill", (long)curwin->w_topfill);
-#endif
+# endif
     dict_add_number(dict, "leftcol", (long)curwin->w_leftcol);
     dict_add_number(dict, "skipcol", (long)curwin->w_skipcol);
 }
