@@ -23,9 +23,6 @@
 #undef TRUE		// will be redefined by exec/types.h
 #undef FALSE
 
-// cproto fails on missing include files, skip them
-#ifndef PROTO
-
 #ifndef LATTICE
 # include <exec/exec.h>
 # include <intuition/intuition.h>
@@ -57,8 +54,6 @@
 # include <libraries/arp_pragmas.h>
 #endif
 
-#endif // PROTO
-
 /*
  * Set stack size to 1 MiB on NG systems. This should be enough even for
  * hungry syntax HL / plugin combinations. Leave the stack alone on OS 3
@@ -302,9 +297,7 @@ mch_init(void)
 #endif
 }
 
-#ifndef PROTO
-# include <workbench/startup.h>
-#endif
+#include <workbench/startup.h>
 
 /*
  * Check_win checks whether we have an interactive window.
@@ -1040,9 +1033,7 @@ mch_settmode(tmode_T tmode)
  * Heavely modified by mool.
  */
 
-#ifndef PROTO
-# include <devices/conunit.h>
-#endif
+#include <devices/conunit.h>
 
 /*
  * Get console size in a system friendly way on AROS and MorphOS.
@@ -1103,21 +1094,21 @@ mch_get_shellsize(void)
 mch_get_shellsize(void)
 {
     struct ConUnit  *conUnit;
-#ifndef __amigaos4__
+# ifndef __amigaos4__
     char	    id_a[sizeof(struct InfoData) + 3];
-#endif
+# endif
     struct InfoData *id=0;
 
     if (!term_console)	// not an amiga window
 	goto out;
 
     // insure longword alignment
-#ifdef __amigaos4__
+# ifdef __amigaos4__
     if (!(id = AllocDosObject(DOS_INFODATA, 0)))
 	goto out;
-#else
+# else
     id = (struct InfoData *)(((long)id_a + 3L) & ~3L);
-#endif
+# endif
 
     /*
      * Should make console aware of real window size, not the one we set.
@@ -1158,9 +1149,9 @@ mch_get_shellsize(void)
 
     return OK;
 out:
-#ifdef __amigaos4__
+# ifdef __amigaos4__
     FreeDosObject(DOS_INFODATA, id); // Safe to pass NULL
-#endif
+# endif
 
     return FAIL;
 }
@@ -1215,11 +1206,9 @@ out_num(long n)
  * say 'oml lib:amiga.lib -r sendpacket.o'
  */
 
-#ifndef PROTO
-// #include <proto/exec.h>
-// #include <proto/dos.h>
+//#include <proto/exec.h>
+//#include <proto/dos.h>
 # include <exec/memory.h>
-#endif
 
 /*
  * Function - dos_packet written by Phil Lindsay, Carolyn Scheppner, and Andy
@@ -1503,7 +1492,7 @@ mch_breakcheck(int force UNUSED)
 // is zero).  Since we want to check for our own ^C's
 
 #ifdef _DCC
-#define Chk_Abort chkabort
+# define Chk_Abort chkabort
 #endif
 
 #ifdef LATTICE
@@ -1695,11 +1684,11 @@ mch_has_wildcard(char_u *p)
 	    ++p;
 	else
 	    if (vim_strchr((char_u *)
-#  ifdef VIM_BACKTICK
+#ifdef VIM_BACKTICK
 				    "*?[(#$`"
-#  else
+#else
 				    "*?[(#$"
-#  endif
+#endif
 						, *p) != NULL
 		    || (*p == '~' && p[1] != NUL))
 		return TRUE;
